package mmm.deco;


import java.util.List;

import mmm.Mmm;
import mmm.utils.I_UBlockSeat;
import mmm.utils.I_URecipeRegistrar;
import mmm.utils.UMaths;
import mmm.utils.USeat;
import net.minecraft.block.Block;
import net.minecraft.block.BlockHorizontal;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.EnumPushReaction;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyDirection;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.Mirror;
import net.minecraft.util.Rotation;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;



public class DBlockChair
		extends Block
		implements I_URecipeRegistrar , I_UBlockSeat
{
	public final static String ID = "deco_chair_";
	public final static String NAME = Mmm.PREFIX + DBlockChair.ID;

	public static final PropertyDirection FACING = BlockHorizontal.FACING;

	private static final AxisAlignedBB BOUNDING_BOX = UMaths.makeBlockAABB( 1 , 0 , 1 , 15 , 16 , 15 );
	private static final AxisAlignedBB COLLISION_BOTTOM = UMaths.makeBlockAABB( 1 , 0 , 1 , 15 , 8 , 15 );
	private static final AxisAlignedBB COLLISION_TOP_NORTH = UMaths.makeBlockAABB( 2 , 8 , 2 , 14 , 16 , 4 );
	private static final AxisAlignedBB COLLISION_TOP_EAST = UMaths.makeBlockAABB( 12 , 8 , 2 , 14 , 16 , 14 );
	private static final AxisAlignedBB COLLISION_TOP_SOUTH = UMaths.makeBlockAABB( 2 , 8 , 12 , 14 , 16 , 14 );
	private static final AxisAlignedBB COLLISION_TOP_WEST = UMaths.makeBlockAABB( 2 , 8 , 2 , 4 , 16 , 14 );

	public final E_DWoodType type;


	public DBlockChair( final E_DWoodType type )
	{
		super( Material.WOOD , type.mapColor );
		this.type = type;
		this.setDefaultState( this.blockState.getBaseState( ).withProperty( DBlockChair.FACING , EnumFacing.NORTH ) );

		this.setCreativeTab( CreativeTabs.DECORATIONS );
		this.setRegistryName( DBlockChair.ID + type.suffix );
		this.setUnlocalizedName( DBlockChair.NAME + type.suffix );

		this.lightOpacity = 0;
		this.translucent = false;
		this.fullBlock = false;
		this.blockHardness = 2.5f;
		this.blockResistance = 12.5f;
		this.blockSoundType = SoundType.LADDER;
		this.enableStats = false;

		this.setHarvestLevel( "axe" , 0 );
	}


	@Override
	public AxisAlignedBB getBoundingBox( final IBlockState state , final IBlockAccess source , final BlockPos pos )
	{
		return DBlockChair.BOUNDING_BOX;
	}


	@Override
	public void addCollisionBoxToList( final IBlockState state , final World worldIn , final BlockPos pos ,
			final AxisAlignedBB container , final List< AxisAlignedBB > output , final Entity entity )
	{
		Block.addCollisionBoxToList( pos , container , output , DBlockChair.COLLISION_BOTTOM );

		AxisAlignedBB back;
		switch ( state.getValue( DBlockChair.FACING ) ) {
			case EAST:
				back = DBlockChair.COLLISION_TOP_EAST;
				break;
			case NORTH:
				back = DBlockChair.COLLISION_TOP_NORTH;
				break;
			case SOUTH:
				back = DBlockChair.COLLISION_TOP_SOUTH;
				break;
			case WEST:
				back = DBlockChair.COLLISION_TOP_WEST;
				break;
			default:
				// TODO log problem
				return;

		}
		Block.addCollisionBoxToList( pos , container , output , back );
	}


	@Override
	public EnumPushReaction getMobilityFlag( final IBlockState state )
	{
		return EnumPushReaction.DESTROY;
	}


	@Override
	public void registerRecipe( )
	{
		GameRegistry.addShapedRecipe( new ItemStack( this ) , //
				"B " , //
				"BB" , //
				"SS" , //
				'B' , new ItemStack( this.type.slab , 1 , this.type.metaData ) , //
				'S' , Items.STICK //
		);
	}


	@Override
	public boolean isOpaqueCube( final IBlockState state )
	{
		return false;
	}


	@Override
	public boolean isFullCube( final IBlockState state )
	{
		return false;
	}


	@Override
	@SideOnly( Side.CLIENT )
	public BlockRenderLayer getBlockLayer( )
	{
		return BlockRenderLayer.CUTOUT;
	}


	@Override
	protected BlockStateContainer createBlockState( )
	{
		return new BlockStateContainer( this , new IProperty[] {
				DBlockChair.FACING
		} );
	}


	@Override
	public IBlockState withRotation( final IBlockState state , final Rotation rot )
	{
		return state.withProperty( DBlockChair.FACING , rot.rotate( state.getValue( DBlockChair.FACING ) ) );
	}


	@Override
	public IBlockState withMirror( final IBlockState state , final Mirror mirrorIn )
	{
		return state.withRotation( mirrorIn.toRotation( state.getValue( DBlockChair.FACING ) ) );
	}


	@Override
	public IBlockState getStateFromMeta( final int meta )
	{
		EnumFacing enumfacing = EnumFacing.getFront( meta );

		if ( enumfacing.getAxis( ) == EnumFacing.Axis.Y ) {
			enumfacing = EnumFacing.NORTH;
		}

		return this.getDefaultState( ).withProperty( DBlockChair.FACING , enumfacing );
	}


	@Override
	public int getMetaFromState( final IBlockState state )
	{
		return state.getValue( DBlockChair.FACING ).getIndex( );
	}


	@Override
	public IBlockState onBlockPlaced( final World worldIn , final BlockPos pos , final EnumFacing facing ,
			final float hitX , final float hitY , final float hitZ , final int meta , final EntityLivingBase placer )
	{
		return this.getDefaultState( ).withProperty( DBlockChair.FACING , placer.getHorizontalFacing( ) );
	}


	@Override
	public boolean onBlockActivated( final World worldIn , final BlockPos pos , final IBlockState state ,
			final EntityPlayer playerIn , final EnumHand hand , final ItemStack heldItem , final EnumFacing side ,
			final float hitX , final float hitY , final float hitZ )
	{
		return USeat.sit( worldIn , pos , playerIn , 0.25 );
	}


	@Override
	public void neighborChanged( final IBlockState state , final World worldIn , final BlockPos pos ,
			final Block blockIn )
	{
		if ( !this.checkSupportBlock( worldIn , pos ) ) {
			this.dropBlockAsItem( worldIn , pos , state , 0 );
			worldIn.setBlockToAir( pos );
		}
	}


	@Override
	public boolean canPlaceBlockAt( final World worldIn , final BlockPos pos )
	{
		return super.canPlaceBlockAt( worldIn , pos ) && this.checkSupportBlock( worldIn , pos );
	}


	private boolean checkSupportBlock( final IBlockAccess worldIn , final BlockPos pos )
	{
		final BlockPos down = pos.down( );
		final IBlockState downState = worldIn.getBlockState( down );
		if ( downState.isSideSolid( worldIn , down , EnumFacing.UP ) ) {
			return true;
		}

		final Block downBlock = downState.getBlock( );
		return downBlock == Blocks.GLASS || downBlock == Blocks.STAINED_GLASS;
	}
}
